/******************************************************************************
版权所有：	通讯项目组
文件名称：	GOOSE.h
文件版本：	V1.00
创建作者：	Mr.L
创建日期：	2014-5-26
功能说明：	GOOSE在通信机中的应用 
其它说明：	
修改记录：	
*/
#ifndef _APP_GOOSE_H
#define _APP_GOOSE_H

#include <app_61850.h>

#ifdef INCLUDE_GOOSE_PTL

/*GOOSE消息类型码*/
#define GOOSE_DATA_TYPE_BI           0x01
#define GOOSE_DATA_TYPE_HJS          0x02
#define GOOSE_DATA_TYPE_LJS          0x04
#define GOOSE_DATA_TYPE_BS           0x08
#define GOOSE_DATA_TYPE_SYN          0x05
#define GOOSE_DATA_TYPE_AJS          0x06
#define GOOSE_DATA_TYPE_ABS          0x07


int app_goose_init(int (*goose_send)(int, const char *, int));
void goose_pub_init(char *gse_data, int gseoutnum);
void app_goose_exit(void);
int app_goose_recv_data_handle(unsigned int index, char *buf, const int buflen, char *outdata);
void app_goose_timer_data_handle(const uint32_t us_diff);
void app_goose_pub_data_handle(char *gse_data, int gseoutnum);
void app_goose_inf_print(void);
#endif // INCLUDE_GOOSE_PTL

#endif // _APP_GOOSE_H
/*--------------------------------- 文件结束 ----------------------------------
*/
